@@ -2,7 +2,7 @@
 /*
  * Integrity volume handling
  *
- * Copyright (C) 2016-2024 Milan Broz
+ * Copyright (C) 2016-2025 Milan Broz
  */
 
 #include <errno.h>
@@ -31,16 +31,22 @@ static int INTEGRITY_read_superblock(struct crypt_device *cd,
 {
 	int devfd, r;
 
+	log_dbg(cd, "Reading kernel dm-integrity metadata on %s.", device_path(device));
+
 	devfd = device_open(cd, device, O_RDONLY);
 	if(devfd < 0)
 		return -EINVAL;
 
 	if (read_lseek_blockwise(devfd, device_block_size(cd, device),
-	    device_alignment(device), sb, sizeof(*sb), offset) != sizeof(*sb) ||
-	    memcmp(sb->magic, SB_MAGIC, sizeof(sb->magic))) {
+	    device_alignment(device), sb, sizeof(*sb), offset) != sizeof(*sb)) {
+		log_dbg(cd, "Cannot read kernel dm-integrity metadata on %s.", device_path(device));
+		return -EINVAL;
+	}
+
+	if (memcmp(sb->magic, SB_MAGIC, sizeof(sb->magic))) {
 		log_dbg(cd, "No kernel dm-integrity metadata detected on %s.", device_path(device));
 		r = -EINVAL;
-	} else if (sb->version < SB_VERSION_1 || sb->version > SB_VERSION_5) {
+	} else if (sb->version < SB_VERSION_1 || sb->version > SB_VERSION_6) {
 		log_err(cd, _("Incompatible kernel dm-integrity metadata (version %u) detected on %s."),
 			sb->version, device_path(device));
 		r = -EINVAL;
@@ -67,8 +73,10 @@ int INTEGRITY_read_sb(struct crypt_device *cd,
 	if (r)
 		return r;
 
-	params->sector_size = SECTOR_SIZE << sb.log2_sectors_per_block;
-	params->tag_size = sb.integrity_tag_size;
+	if (params) {
+		params->sector_size = SECTOR_SIZE << sb.log2_sectors_per_block;
+		params->tag_size = sb.integrity_tag_size;
+	}
 
 	if (flags)
 		*flags = sb.flags;
@@ -79,28 +87,32 @@ int INTEGRITY_read_sb(struct crypt_device *cd,
 int INTEGRITY_dump(struct crypt_device *cd, struct device *device, uint64_t offset)
 {
 	struct superblock sb;
+	uint64_t sector_size;
 	int r;
 
 	r = INTEGRITY_read_superblock(cd, device, offset, &sb);
 	if (r)
 		return r;
 
-	log_std(cd, "Info for integrity device %s.\n", device_path(device));
-	log_std(cd, "superblock_version %d\n", (unsigned)sb.version);
-	log_std(cd, "log2_interleave_sectors %d\n", sb.log2_interleave_sectors);
-	log_std(cd, "integrity_tag_size %u\n", sb.integrity_tag_size);
-	log_std(cd, "journal_sections %u\n", sb.journal_sections);
-	log_std(cd, "provided_data_sectors %" PRIu64 "\n", sb.provided_data_sectors);
-	log_std(cd, "sector_size %u\n", SECTOR_SIZE << sb.log2_sectors_per_block);
+	sector_size = (uint64_t)SECTOR_SIZE << sb.log2_sectors_per_block;
+	log_std(cd, "INTEGRITY header information for %s.\n", device_path(device));
+	log_std(cd, "version: %d\n", (unsigned)sb.version);
+	log_std(cd, "tag size: %u [bytes]\n", sb.integrity_tag_size);
+	log_std(cd, "sector size: %" PRIu64 " [bytes]\n", sector_size);
+	log_std(cd, "data size: %" PRIu64 " [512-byte units] (%" PRIu64 " [bytes])\n",
+		sb.provided_data_sectors, sb.provided_data_sectors * SECTOR_SIZE);
 	if (sb.version >= SB_VERSION_2 && (sb.flags & SB_FLAG_RECALCULATING))
-		log_std(cd, "recalc_sector %" PRIu64 "\n", sb.recalc_sector);
-	log_std(cd, "log2_blocks_per_bitmap %u\n", sb.log2_blocks_per_bitmap_bit);
-	log_std(cd, "flags %s%s%s%s%s\n",
+		log_std(cd, "recalculate sector: %" PRIu64 "\n", sb.recalc_sector);
+	log_std(cd, "journal sections: %u\n", sb.journal_sections);
+	log_std(cd, "log2 interleave sectors: %d\n", sb.log2_interleave_sectors);
+	log_std(cd, "log2 blocks per bitmap: %u\n", sb.log2_blocks_per_bitmap_bit);
+	log_std(cd, "flags: %s%s%s%s%s%s\n",
 		sb.flags & SB_FLAG_HAVE_JOURNAL_MAC ? "have_journal_mac " : "",
 		sb.flags & SB_FLAG_RECALCULATING ? "recalculating " : "",
 		sb.flags & SB_FLAG_DIRTY_BITMAP ? "dirty_bitmap " : "",
 		sb.flags & SB_FLAG_FIXED_PADDING ? "fix_padding " : "",
-		sb.flags & SB_FLAG_FIXED_HMAC ? "fix_hmac " : "");
+		sb.flags & SB_FLAG_FIXED_HMAC ? "fix_hmac " : "",
+		sb.flags & SB_FLAG_INLINE ? "inline " : "");
 
 	return 0;
 }
@@ -120,26 +132,36 @@ int INTEGRITY_data_sectors(struct crypt_device *cd,
 	return 0;
 }
 
-int INTEGRITY_key_size(const char *integrity)
+int INTEGRITY_key_size(const char *integrity, int required_key_size)
 {
+	int ks = 0;
+
+	if (!integrity && required_key_size)
+		return -EINVAL;
+
 	if (!integrity)
 		return 0;
 
 	//FIXME: use crypto backend hash size
 	if (!strcmp(integrity, "aead"))
-		return 0;
+		ks = 0;
 	else if (!strcmp(integrity, "hmac(sha1)"))
-		return 20;
+		ks = required_key_size ?: 20;
 	else if (!strcmp(integrity, "hmac(sha256)"))
-		return 32;
+		ks = required_key_size ?: 32;
 	else if (!strcmp(integrity, "hmac(sha512)"))
-		return 64;
+		ks = required_key_size ?: 64;
 	else if (!strcmp(integrity, "poly1305"))
-		return 0;
+		ks = 0;
 	else if (!strcmp(integrity, "none"))
-		return 0;
+		ks = 0;
+	else
+		return -EINVAL;
 
-	return -EINVAL;
+	if (required_key_size && ks != required_key_size)
+		return -EINVAL;
+
+	return ks;
 }
 
 /* Return hash or hmac(hash) size, if known */
@@ -230,6 +252,9 @@ int INTEGRITY_create_dmd_device(struct crypt_device *cd,
 	if (sb_flags & SB_FLAG_RECALCULATING)
 		dmd->flags |= CRYPT_ACTIVATE_RECALCULATE;
 
+	if (sb_flags & SB_FLAG_INLINE)
+		dmd->flags |= (CRYPT_ACTIVATE_NO_JOURNAL | CRYPT_ACTIVATE_INLINE_MODE);
+
 	r = INTEGRITY_data_sectors(cd, INTEGRITY_metadata_device(cd),
 				   crypt_get_data_offset(cd) * SECTOR_SIZE, &dmd->size);
 	if (r < 0)
@@ -249,14 +274,15 @@ int INTEGRITY_activate_dmd_device(struct crypt_device *cd,
 		       uint32_t sb_flags)
 {
 	int r;
-	uint32_t dmi_flags;
+	uint64_t dmi_flags;
 	struct dm_target *tgt = &dmd->segment;
 
 	if (!single_segment(dmd) || tgt->type != DM_INTEGRITY)
 		return -EINVAL;
 
-	log_dbg(cd, "Trying to activate INTEGRITY device on top of %s, using name %s, tag size %d, provided sectors %" PRIu64".",
-		device_path(tgt->data_device), name, tgt->u.integrity.tag_size, dmd->size);
+	log_dbg(cd, "Trying to activate INTEGRITY device on top of %s, using name %s, tag size %d%s, provided sectors %" PRIu64".",
+		device_path(tgt->data_device), name, tgt->u.integrity.tag_size,
+		(sb_flags & SB_FLAG_INLINE) ? " (inline)" :"", dmd->size);
 
 	r = create_or_reload_device(cd, name, type, dmd);
 
@@ -280,6 +306,12 @@ int INTEGRITY_activate_dmd_device(struct crypt_device *cd,
 		return -ENOTSUP;
 	}
 
+	if (r < 0 && (sb_flags & SB_FLAG_INLINE) && !dm_flags(cd, DM_INTEGRITY, &dmi_flags) &&
+	    !(dmi_flags & DM_INTEGRITY_INLINE_MODE_SUPPORTED)) {
+		log_err(cd, _("Kernel does not support dm-integrity inline mode."));
+		return -ENOTSUP;
+	}
+
 	return r;
 }
 
@@ -372,11 +404,14 @@ static int _create_reduced_device(struct crypt_device *cd,
 
 int INTEGRITY_format(struct crypt_device *cd,
 		     const struct crypt_params_integrity *params,
+		     struct volume_key *integrity_key,
 		     struct volume_key *journal_crypt_key,
 		     struct volume_key *journal_mac_key,
-		     uint64_t backing_device_sectors)
+		     uint64_t backing_device_sectors,
+		     uint32_t *sb_flags,
+		     bool integrity_inline)
 {
-	uint32_t dmi_flags;
+	uint64_t dmi_flags;
 	char reduced_device_name[70], tmp_name[64], tmp_uuid[40];
 	struct crypt_dm_active_device dmdi = {
 		.size = 8,
@@ -387,7 +422,6 @@ int INTEGRITY_format(struct crypt_device *cd,
 	uuid_t tmp_uuid_bin;
 	uint64_t data_offset_sectors;
 	struct device *p_metadata_device, *p_data_device, *reduced_device = NULL;
-	struct volume_key *vk = NULL;
 
 	uuid_generate(tmp_uuid_bin);
 	uuid_unparse(tmp_uuid_bin, tmp_uuid);
@@ -422,19 +456,18 @@ int INTEGRITY_format(struct crypt_device *cd,
 		p_data_device = crypt_data_device(cd);
 	}
 
-	/* There is no data area, we can actually use fake zeroed key */
-	if (params && params->integrity_key_size)
-		vk = crypt_alloc_volume_key(params->integrity_key_size, NULL);
+	if (integrity_inline)
+		dmdi.flags |= (CRYPT_ACTIVATE_NO_JOURNAL | CRYPT_ACTIVATE_INLINE_MODE);
 
 	r = dm_integrity_target_set(cd, tgt, 0, dmdi.size, p_metadata_device,
 			p_data_device, crypt_get_integrity_tag_size(cd),
-			data_offset_sectors, crypt_get_sector_size(cd), vk,
+			data_offset_sectors, crypt_get_sector_size(cd), integrity_key,
 			journal_crypt_key, journal_mac_key, params);
 	if (r < 0)
 		goto err;
 
-	log_dbg(cd, "Trying to format INTEGRITY device on top of %s, tmp name %s, tag size %d.",
-		device_path(tgt->data_device), tmp_name, tgt->u.integrity.tag_size);
+	log_dbg(cd, "Trying to format INTEGRITY device on top of %s, tmp name %s, tag size %d%s.",
+		device_path(tgt->data_device), tmp_name, tgt->u.integrity.tag_size, integrity_inline ? " (inline)" : "");
 
 	r = device_block_adjust(cd, tgt->data_device, DEV_EXCL, tgt->u.integrity.offset, NULL, NULL);
 	if (r < 0 && (dm_flags(cd, DM_INTEGRITY, &dmi_flags) || !(dmi_flags & DM_INTEGRITY_SUPPORTED))) {
@@ -455,9 +488,14 @@ int INTEGRITY_format(struct crypt_device *cd,
 		goto err;
 
 	r = dm_remove_device(cd, tmp_name, CRYPT_DEACTIVATE_FORCE);
+	if (r)
+		goto err;
+
+	/* reload sb_flags from superblock (important for SB_FLAG_INLINE) */
+	if (sb_flags)
+		r = INTEGRITY_read_sb(cd, NULL, sb_flags);
 err:
 	dm_targets_free(cd, &dmdi);
-	crypt_free_volume_key(vk);
 	if (reduced_device) {
 		dm_remove_device(cd, reduced_device_name, CRYPT_DEACTIVATE_FORCE);
 		device_free(cd, reduced_device);
