@@ -2,11 +2,12 @@
 /*
  * LUKS - Linux Unified Key Setup v2, keyslot handling
  *
- * Copyright (C) 2015-2024 Red Hat, Inc. All rights reserved.
- * Copyright (C) 2015-2024 Milan Broz
+ * Copyright (C) 2015-2025 Red Hat, Inc. All rights reserved.
+ * Copyright (C) 2015-2025 Milan Broz
  */
 
 #include "luks2_internal.h"
+#include "keyslot_context.h"
 
 /* Internal implementations */
 extern const keyslot_handler luks2_keyslot;
@@ -75,8 +76,38 @@ int LUKS2_keyslot_find_empty(struct crypt_device *cd, struct luks2_hdr *hdr, siz
 /* Check if a keyslot is assigned to specific segment */
 static int _keyslot_for_segment(struct luks2_hdr *hdr, int keyslot, int segment)
 {
-	int keyslot_digest, count = 0;
+	json_object *jobj_keyslots, *jobj;
+	crypt_keyslot_priority slot_priority;
 	unsigned s;
+	int keyslot_digest, count = 0;
+
+	/*
+	 * Must not be called with both keyslot == CRYPT_ANY_SLOT
+	 * and segment == CRYPT_ONE_SEGMENT. The CRYPT_DEFAULT_SEGMENT
+	 * and CRYPT_ANY_SEGMENT are handled properly in upper layer.
+	 */
+	assert(keyslot >= 0 || segment >= 0);
+
+	if (keyslot == CRYPT_ANY_SLOT) {
+		json_object_object_get_ex(hdr->jobj, "keyslots", &jobj_keyslots);
+
+		json_object_object_foreach(jobj_keyslots, slot, val) {
+			if (!json_object_object_get_ex(val, "priority", &jobj))
+				slot_priority = CRYPT_SLOT_PRIORITY_NORMAL;
+			else
+				slot_priority = json_object_get_int(jobj);
+
+			if (slot_priority < CRYPT_SLOT_PRIORITY_NORMAL)
+				continue;
+
+			keyslot_digest = LUKS2_digest_by_keyslot(hdr, atoi(slot));
+			if (keyslot_digest >= 0 &&
+			    keyslot_digest == LUKS2_digest_by_segment(hdr, segment))
+				return 1;
+		}
+
+		return 0;
+	}
 
 	keyslot_digest = LUKS2_digest_by_keyslot(hdr, keyslot);
 	if (keyslot_digest < 0)
@@ -93,16 +124,6 @@ static int _keyslot_for_segment(struct luks2_hdr *hdr, int keyslot, int segment)
 	return count;
 }
 
-static int _keyslot_for_digest(struct luks2_hdr *hdr, int keyslot, int digest)
-{
-	int r = -EINVAL;
-
-	r = LUKS2_digest_by_keyslot(hdr, keyslot);
-	if (r < 0)
-		return r;
-	return r == digest ? 0 : -ENOENT;
-}
-
 int LUKS2_keyslot_for_segment(struct luks2_hdr *hdr, int keyslot, int segment)
 {
 	int r = -EINVAL;
@@ -147,6 +168,16 @@ int LUKS2_keyslot_cipher_incompatible(struct crypt_device *cd, const char *ciphe
 	if (!cipher_spec || crypt_is_cipher_null(cipher_spec))
 		return 1;
 
+	/*
+	 * Do not allow capi format for keyslots
+	 * Note: It always failed in ivsize check later anyway.
+	 */
+	if (!strncmp(cipher_spec, "capi:", 5))
+		return 1;
+
+	if (crypt_is_cipher_null(cipher_spec))
+		return 1;
+
 	if (crypt_parse_name_and_mode(cipher_spec, cipher, NULL, cipher_mode) < 0)
 		return 1;
 
@@ -318,61 +349,43 @@ static int _open_and_verify(struct crypt_device *cd,
 	int keyslot,
 	const char *password,
 	size_t password_len,
-	struct volume_key **vk)
+	struct volume_key **r_vk)
 {
 	int r, key_size = LUKS2_get_keyslot_stored_key_size(hdr, keyslot);
+	struct volume_key *vk = NULL;
+	void *key = NULL;
 
 	if (key_size < 0)
 		return -EINVAL;
 
-	*vk = crypt_alloc_volume_key(key_size, NULL);
-	if (!*vk)
+	key = crypt_safe_alloc(key_size);
+	if (!key)
 		return -ENOMEM;
 
-	r = h->open(cd, keyslot, password, password_len, (*vk)->key, (*vk)->keylength);
-	if (r < 0)
-		log_dbg(cd, "Keyslot %d (%s) open failed with %d.", keyslot, h->name, r);
-	else
-		r = LUKS2_digest_verify(cd, hdr, *vk, keyslot);
-
+	r = h->open(cd, keyslot, password, password_len, key, key_size);
 	if (r < 0) {
-		crypt_free_volume_key(*vk);
-		*vk = NULL;
+		log_dbg(cd, "Keyslot %d (%s) open failed with %d.", keyslot, h->name, r);
+		goto err;
 	}
 
-	crypt_volume_key_set_id(*vk, r);
-
-	return r < 0 ? r : keyslot;
-}
-
-static int LUKS2_open_and_verify_by_digest(struct crypt_device *cd,
-	struct luks2_hdr *hdr,
-	int keyslot,
-	int digest,
-	const char *password,
-	size_t password_len,
-	struct volume_key **vk)
-{
-	const keyslot_handler *h;
-	int r;
-
-	if (!(h = LUKS2_keyslot_handler(cd, keyslot)))
-		return -ENOENT;
-
-	r = h->validate(cd, LUKS2_get_keyslot_jobj(hdr, keyslot));
-	if (r) {
-		log_dbg(cd, "Keyslot %d validation failed.", keyslot);
-		return r;
+	vk = crypt_alloc_volume_key_by_safe_alloc(&key);
+	if (!vk) {
+		r = -ENOMEM;
+		goto err;
 	}
 
-	r = _keyslot_for_digest(hdr, keyslot, digest);
-	if (r) {
-		if (r == -ENOENT)
-			log_dbg(cd, "Keyslot %d unusable for digest %d.", keyslot, digest);
-		return r;
-	}
+	r = LUKS2_digest_verify(cd, hdr, vk, keyslot);
+	if (r < 0)
+		goto err;
 
-	return _open_and_verify(cd, hdr, h, keyslot, password, password_len, vk);
+	crypt_volume_key_set_id(vk, r);
+	*r_vk = vk;
+	return keyslot;
+err:
+	crypt_safe_free(key);
+	crypt_free_volume_key(vk);
+
+	return r;
 }
 
 static int LUKS2_open_and_verify(struct crypt_device *cd,
@@ -405,49 +418,6 @@ static int LUKS2_open_and_verify(struct crypt_device *cd,
 	return _open_and_verify(cd, hdr, h, keyslot, password, password_len, vk);
 }
 
-static int LUKS2_keyslot_open_priority_digest(struct crypt_device *cd,
-	struct luks2_hdr *hdr,
-	crypt_keyslot_priority priority,
-	const char *password,
-	size_t password_len,
-	int digest,
-	struct volume_key **vk)
-{
-	json_object *jobj_keyslots, *jobj;
-	crypt_keyslot_priority slot_priority;
-	int keyslot, r = -ENOENT, r_old;
-
-	json_object_object_get_ex(hdr->jobj, "keyslots", &jobj_keyslots);
-
-	json_object_object_foreach(jobj_keyslots, slot, val) {
-		r_old = r;
-
-		if (!json_object_object_get_ex(val, "priority", &jobj))
-			slot_priority = CRYPT_SLOT_PRIORITY_NORMAL;
-		else
-			slot_priority = json_object_get_int(jobj);
-
-		keyslot = atoi(slot);
-		if (slot_priority != priority) {
-			log_dbg(cd, "Keyslot %d priority %d != %d (required), skipped.",
-				keyslot, slot_priority, priority);
-			continue;
-		}
-
-		r = LUKS2_open_and_verify_by_digest(cd, hdr, keyslot, digest, password, password_len, vk);
-
-		/* Do not retry for errors that are no -EPERM or -ENOENT,
-		   former meaning password wrong, latter key slot unusable for segment */
-		if ((r != -EPERM) && (r != -ENOENT))
-			break;
-		/* If a previous keyslot failed with EPERM (bad password) prefer it */
-		if (r_old == -EPERM && r == -ENOENT)
-			r = -EPERM;
-	}
-
-	return r;
-}
-
 static int LUKS2_keyslot_open_priority(struct crypt_device *cd,
 	struct luks2_hdr *hdr,
 	crypt_keyslot_priority priority,
@@ -491,70 +461,83 @@ static int LUKS2_keyslot_open_priority(struct crypt_device *cd,
 	return r;
 }
 
-static int LUKS2_keyslot_open_by_digest(struct crypt_device *cd,
-	struct luks2_hdr *hdr,
-	int keyslot,
-	int digest,
-	const char *password,
-	size_t password_len,
-	struct volume_key **vk)
-{
-	int r_prio, r = -EINVAL;
-
-	if (digest < 0)
-		return r;
-
-	if (keyslot == CRYPT_ANY_SLOT) {
-		r_prio = LUKS2_keyslot_open_priority_digest(cd, hdr, CRYPT_SLOT_PRIORITY_PREFER,
-			password, password_len, digest, vk);
-		if (r_prio >= 0)
-			r = r_prio;
-		else if (r_prio != -EPERM && r_prio != -ENOENT)
-			r = r_prio;
-		else
-			r = LUKS2_keyslot_open_priority_digest(cd, hdr, CRYPT_SLOT_PRIORITY_NORMAL,
-				password, password_len, digest, vk);
-		/* Prefer password wrong to no entry from priority slot */
-		if (r_prio == -EPERM && r == -ENOENT)
-			r = r_prio;
-	} else
-		r = LUKS2_open_and_verify_by_digest(cd, hdr, keyslot, digest, password, password_len, vk);
-
-	return r;
-}
-
-int LUKS2_keyslot_open_all_segments(struct crypt_device *cd,
+static int keyslot_context_open_all_segments(struct crypt_device *cd,
 	int keyslot_old,
 	int keyslot_new,
-	const char *password,
-	size_t password_len,
-	struct volume_key **vks)
+	struct crypt_keyslot_context *kc_old,
+	struct crypt_keyslot_context *kc_new,
+	struct volume_key **r_vks)
 {
+	int segment_old, segment_new, digest_old = -1, digest_new = -1, r = -ENOENT;
+	struct luks2_hdr *hdr;
 	struct volume_key *vk = NULL;
-	int digest_old, digest_new, r = -EINVAL;
-	struct luks2_hdr *hdr = crypt_get_hdr(cd, CRYPT_LUKS2);
 
-	digest_old = LUKS2_reencrypt_digest_old(hdr);
-	if (digest_old >= 0) {
-		log_dbg(cd, "Trying to unlock volume key (digest: %d) using keyslot %d.", digest_old, keyslot_old);
-		r = LUKS2_keyslot_open_by_digest(cd, hdr, keyslot_old, digest_old, password, password_len, &vk);
+	assert(cd);
+	assert(!kc_old || kc_old->get_luks2_key);
+	assert(!kc_new || kc_new->get_luks2_key);
+	assert(r_vks);
+
+	hdr = crypt_get_hdr(cd, CRYPT_LUKS2);
+
+	segment_old = LUKS2_reencrypt_segment_old(hdr);
+	segment_new = LUKS2_reencrypt_segment_new(hdr);
+
+	if (segment_old < 0 || segment_new < 0)
+		return -EINVAL;
+
+	digest_old = LUKS2_digest_by_segment(hdr, segment_old);
+	digest_new = LUKS2_digest_by_segment(hdr, segment_new);
+
+	if (digest_old >= 0 && digest_new >= 0 && digest_old != digest_new && (!kc_old || !kc_new))
+		return -ESRCH;
+
+	if (digest_old >= 0 && kc_old) {
+		log_dbg(cd, "Checking current volume key (digest %d, segment: %d) using keyslot %d.",
+			    digest_old, segment_old, keyslot_old);
+
+		/* key and key in keyring types do not have association with any keyslot */
+		if (kc_old->type != CRYPT_KC_TYPE_KEY && kc_old->type != CRYPT_KC_TYPE_VK_KEYRING) {
+			r = LUKS2_keyslot_for_segment(hdr, keyslot_old, segment_old);
+			if (r < 0)
+				goto out;
+		}
+
+		r = kc_old->get_luks2_key(cd, kc_old, keyslot_old, segment_old, &vk);
 		if (r < 0)
 			goto out;
-		crypt_volume_key_add_next(vks, vk);
+		crypt_volume_key_add_next(r_vks, vk);
+		if (crypt_volume_key_get_id(vk) < 0 && LUKS2_digest_verify_by_digest(cd, digest_old, vk) == digest_old)
+			crypt_volume_key_set_id(vk, digest_old);
+		if (crypt_volume_key_get_id(vk) != digest_old) {
+			r = -EPERM;
+			goto out;
+		}
 	}
 
-	digest_new = LUKS2_reencrypt_digest_new(hdr);
-	if (digest_new >= 0 && digest_old != digest_new) {
-		log_dbg(cd, "Trying to unlock volume key (digest: %d) using keyslot %d.", digest_new, keyslot_new);
-		r = LUKS2_keyslot_open_by_digest(cd, hdr, keyslot_new, digest_new, password, password_len, &vk);
+	if (digest_new >= 0 && digest_old != digest_new && kc_new) {
+		log_dbg(cd, "Checking new volume key (digest %d, segment: %d) using keyslot %d.",
+			    digest_new, segment_new, keyslot_new);
+
+		/* key and key in keyring types do not have association with any keyslot */
+		if (kc_new->type != CRYPT_KC_TYPE_KEY && kc_new->type != CRYPT_KC_TYPE_VK_KEYRING) {
+			r = LUKS2_keyslot_for_segment(hdr, keyslot_new, segment_new);
+			if (r < 0)
+				goto out;
+		}
+
+		r = kc_new->get_luks2_key(cd, kc_new, keyslot_new, segment_new, &vk);
 		if (r < 0)
 			goto out;
-		crypt_volume_key_add_next(vks, vk);
+		crypt_volume_key_add_next(r_vks, vk);
+		if (crypt_volume_key_get_id(vk) < 0 && LUKS2_digest_verify_by_digest(cd, digest_new, vk) == digest_new)
+			crypt_volume_key_set_id(vk, digest_new);
+		if (crypt_volume_key_get_id(vk) != digest_new)
+			r = -EPERM;
 	}
 out:
 	if (r < 0) {
-		crypt_free_volume_key(*vks);
-		*vks = NULL;
+		crypt_free_volume_key(*r_vks);
+		*r_vks = NULL;
 
 		if (r == -ENOMEM)
 			log_err(cd, _("Not enough available memory to open a keyslot."));
@@ -564,6 +547,25 @@ out:
 	return r;
 }
 
+int LUKS2_keyslot_context_open_all_segments(struct crypt_device *cd,
+	int keyslot1,
+	int keyslot2,
+	struct crypt_keyslot_context *kc1,
+	struct crypt_keyslot_context *kc2,
+	struct volume_key **r_vks)
+{
+	int r, r2;
+
+	r = keyslot_context_open_all_segments(cd, keyslot1, keyslot2, kc1, kc2, r_vks);
+	if (r == -EPERM || r == -ENOENT) {
+		r2 = keyslot_context_open_all_segments(cd, keyslot2, keyslot1, kc2, kc1, r_vks);
+		if (r2 != -ENOENT)
+			r = r2;
+	}
+
+	return r;
+}
+
 int LUKS2_keyslot_open(struct crypt_device *cd,
 	int keyslot,
 	int segment,
@@ -646,7 +648,7 @@ int LUKS2_keyslot_store(struct crypt_device *cd,
 		if (!h)
 			return -EINVAL;
 
-		r = h->alloc(cd, keyslot, vk->keylength, params);
+		r = h->alloc(cd, keyslot, crypt_volume_key_length(vk), params);
 		if (r)
 			return r;
 	} else {
@@ -670,7 +672,7 @@ int LUKS2_keyslot_store(struct crypt_device *cd,
 		return -EINVAL;
 
 	return h->store(cd, keyslot, password, password_len,
-			vk->key, vk->keylength);
+			crypt_volume_key_get_key(vk), crypt_volume_key_length(vk));
 }
 
 int LUKS2_keyslot_wipe(struct crypt_device *cd,
@@ -861,8 +863,7 @@ int LUKS2_keyslots_validate(struct crypt_device *cd, json_object *hdr_jobj)
 	if (!json_object_object_get_ex(hdr_jobj, "keyslots", &jobj_keyslots))
 		return -EINVAL;
 
-	if (LUKS2_config_get_requirements(cd, &dummy, &reqs))
-		return -EINVAL;
+	LUKS2_config_get_requirements(cd, &dummy, &reqs);
 
 	json_object_object_foreach(jobj_keyslots, slot, val) {
 		keyslot = atoi(slot);
