@@ -15,7 +15,7 @@
 # $1 full target dir
 # $2 full source luks2 image
 
-function generate()
+generate()
 {
 	# 128KiB metadata
 	TEST_MDA_SIZE=$LUKS2_HDR_SIZE_128K
@@ -44,7 +44,7 @@ function generate()
 	lib_mangle_json_hdr1 $TEST_MDA_SIZE $TEST_JSN_SIZE kill
 }
 
-function check()
+check()
 {
 	lib_hdr1_killed $TEST_MDA_SIZE || exit 2
 
