@@ -15,7 +15,7 @@
 # $1 full target dir
 # $2 full source luks2 image
 
-function generate()
+generate()
 {
 	# 4 MiB metadata
 	TEST_MDA_SIZE=$LUKS2_HDR_SIZE_4M
@@ -46,7 +46,7 @@ function generate()
 	lib_mangle_json_hdr1 $TEST_MDA_SIZE $TEST_JSN_SIZE
 }
 
-function check()
+check()
 {
 	lib_hdr0_killed $TEST_MDA_SIZE || exit 2
 
